@@ -4,7 +4,7 @@
  *  Linux driver for dec's depca Ethernet adapter family.
  *    not otherwise copyrighted parts (by the contributed
  *    authors and their parents) are  copyrighted by
- *    (C) Peter Bauer, Systematika GmbH (100136,3530@compuserve.com)
+ *    (C) Peter Bauer, Systematika GmbH (100136.3530@compuserve.com)
  *  This driver is based on 
  *    depca.asm version 3 Copyright Russell Nelson, Crynwr Software 
  *    =packet driver
@@ -16,7 +16,13 @@
  *  DEPCA-family ethernet cards. This intention comes from the packet
  *  driver, which also attempts to serve the whole family. However,
  *  for me it was until now only possible to test this driver against 
- *  a DE200 card.
+ *  a DE200 card. Together with information received from others, it
+ *  should work with (24-Jul-93)
+ *   DE200 64K
+ *   DE200 32K
+ *   DE100
+ *   DEPCA rev E
+ *
  *  The whole adapter-programing is deriverd from the packet-driver, and
  *  the whole linux-programing is derived from d_link.c (I used this one
  *  because its a driver and because it drives a ethernet device and 
@@ -57,17 +63,34 @@
  *  - fixed bug in isRom (shoud be <<9 and not <<8) (This will only harm 
  *    those, which have rom installed at the address where the depca wants 
  *    to live)
- *  - fixed bug in adapter_init, which will prevent depca's with address-
+ *  - fixed bug in adapter_init, which will make depca's with address-
  *    bases not at 64K-boundary not to work (This is true i think for
  *    all depca's with less than 64K address-space... :-)
  *  - reduced calls to xlateAndMap to a minimum
+ * Revision 0.5:
+ *  - Supports multiple depcas now. (as proposed by bob@inmos.co.uk)
+ *    (the max will be 2, because the iobase can be either 200 or 300)
+ *    The easiest way to do this was to use classes, so i did so... 
+ *    The side effect is, that it only compiles now with pl11 (pl10 is 
+ *    no longer supported, also i see no reason why c++ specific parts
+ *    could not be linked to pl10). 
+ *    I choosed the irq to be the index to a driver (so no two depcas 
+ *    can live at the same irq), because this is readily available in 
+ *    the interrupt.
+ *    Some of the mechanisms built in here could be moved to a higher layer 
+ *    (baseclass) because they are not specific to a special ethernet driver.
+ *    (e.g. struct device should  become class, then each driver can
+ *    inherit from it, and overload the open/close etc with its specific
+ *    implementation...)
+ *    Also, the structures associated with the lance memory could become
+ *    a class too.
  *  
  * (the revision counter gets ++ each time something gets changed after i have
  *  given a copy to someone)
  */
 
 static char *version =
-  "depca.c: $Revision: 0.4 ALPHA $,  Peter Bauer (100136,3530@compuserve.com)\n";
+  "depca.c: $Revision: 0.5 ALPHA $,  Peter Bauer (100136,3530@compuserve.com)\n";
 
 
 /* First put some garlic to keep the vampires away: */
@@ -129,6 +152,23 @@ static char *version =
 #include "skbuff.h"
 #include "sock.h"
 #include "arp.h"
+/* This is the interface to the rest of the world */
+    /* Put in the device structure. */
+    static int depca_open(struct device *dev);
+    static int depca_close(struct device *dev);
+    static int depca_start_xmit(struct sk_buff *skb, struct device *dev);
+
+    static void depca_interrupt(int reg_ptr);/* Dispatch from interrupts. */
+    int depca_init(struct device *dev); /* Initialization */
+    /* Passed to sigaction() to register the interrupt handler. */
+    static struct sigaction depca_sigaction = {
+      &depca_interrupt,
+      0,
+      0,
+      NULL,
+    };
+
+/* the rest is all local */
 
 static void SlowerOutb (int v, int p) {
   outb_p (v,p);
@@ -146,7 +186,7 @@ static void SlowerOutw (int v, int p) {
 #define outw SlowerOutw
 
 #ifdef ALPHA
-#define BlaBla(bla) {iAm (realdev);printk bla;}
+#define BlaBla(bla) {iAm ();printk bla;}
 #else
 #define BlaBla(sagIchNicht) 
 #endif
@@ -162,54 +202,119 @@ struct netstats {
   int soft_rx_errors;
   int soft_trx_err_bits;
 };
-static struct netstats *localstats;
 
-/*
- * Index to functions, as function prototypes.
- */
+typedef unsigned short uword;
+typedef unsigned char  uchar;
+const maxIRQs=16; /* 0..15 */
 
-/* Put in the device structure. */
-static int depca_open(struct device *dev);
-static int depca_close(struct device *dev);
-static int depca_start_xmit(struct sk_buff *skb, struct device *dev);
-
-/* Dispatch from interrupts. */
-static void depca_interrupt(int reg_ptr);
-static void depca_tx_intr(struct device *dev);
-static void depca_rx_intr(struct device *dev);
-
-/* Initialization */
-int depca_init(struct device *dev);
-static void adapter_init(struct device *dev, int startp);
-
-/* Passed to sigaction() to register the interrupt handler. */
-static struct sigaction depca_sigaction = {
-  &depca_interrupt,
-  0,
-  0,
-  NULL,
+/* new/delete required, because inside kernel. But should move to
+   some library */
+static void *operator new (size_t s) { 
+  return kmalloc(s, GFP_KERNEL);
+};
+static void operator delete (void * p) { 
+  kfree (p);
 };
 
-/*
- * DEPCA driver constants, structs and variables:
- */
+static class depcaDriver /* : public device */ {
+public:
+  depcaDriver (struct device *_dev);
+  int init ();
+  int open ();
+  int close();
+  int startXmit(struct sk_buff *skb);
+  void interrupt ();
+private:
+  void valToPort (uword v,uword reg);
+  void iAm ();
+  uchar* xlateAndMap (uword a);
+  int getXmitBuf (void);
+  void txIntr();
+  void rxIntr();
+  int detectBoard (void);
+  char *checkSignature (uchar * p);
+  int initialize (void);
+  void initializeMulti (int v);
+  int initializeNoMulti (int v);
+  int receiveMode (enum receiveModeType r);
+  void adapterInit(int startp);
 
-#define uword unsigned short
-#define uchar unsigned char
+  struct netstats*& localstats;
+  uword receiveHead;  /* ptr next packet to be filled by LANCE. */
+  uword transmitHead; /* ptr next packet to be filled by host. */
+  uchar adapterInitOk;
+  struct device *dev;
+  uchar niCsrValue;
+  enum receiveModeType {disaReceiveXmit,noMulticast,multicast,promiscuous};
+  enum {translate_64k,translate_32k,translate_2k} xlatemode;
+  int ints;
+};
 
+static int drvInit=0; /* flag to clear the drv-array */
+static depcaDriver *depcaDrivers[maxIRQs];
+
+/* This is the implementation of the interface-routines */
+  /* create new object to serve the dev's irq */
+  int depca_init (struct device *dev) {
+    if (!drvInit) {
+      drvInit=1;
+      for (int i=0;i<maxIRQs;i++)
+        depcaDrivers[i]=NULL;
+    }
+    depcaDriver ** thisOne=&depcaDrivers[dev->irq];
+    if (*thisOne) {
+      printk ("depca_init called for irq allready inuse by depca");
+      delete *thisOne;
+    }
+    *thisOne=new depcaDriver (dev);
+    return (*thisOne)->init();
+  }
+
+  /* Open/initialize the board.  This is called (in the current kernel)
+   * sometime after booting when the 'config <dev->name>' program is run.
+   *
+   * This routine should set everything up anew at each open, even
+   * registers that "should" only need to be set once at boot, so that
+   * there is a non-reboot way to recover if something goes wrong.
+   */
+  static int depca_open(struct device *dev) {
+    return depcaDrivers[dev->irq]->open();
+  }
+
+  /* The inverse routine to depca_open().  */
+  static int depca_close(struct device *dev) {
+    return depcaDrivers[dev->irq]->close();
+  }
+
+  /* This gets called by the interrupt logic of the kernel */
+  static void depca_interrupt(int reg_ptr) {
+    int irq = -(((struct pt_regs *)reg_ptr)->orig_eax+2);
+    if (depcaDrivers [irq])
+      depcaDrivers [irq]->interrupt();
+    else
+      printk ("?: DEPCA: irq %d for unknown device.\n", irq);
+  }
+
+  /* Copy a buffer to the adapter transmit page memory. Start sending. */
+  static int depca_start_xmit(struct sk_buff *skb, struct device *dev) {
+    return depcaDrivers[dev->irq]->startXmit (skb);
+  }
+
+
+/* DEPCA driver constants, structs and variables */
 #define RUNT    60
 #define GIANT 1514 /* largest legal size packet, no fcs */
 
-#define NI_CSR     (0+realdev->base_addr)
-#define RBI        (2+realdev->base_addr)
-#define DATA_REG   (4+realdev->base_addr)
-#define ADDR_REG   (6+realdev->base_addr)
+#define NI_CSR     (0+dev->base_addr)
+#define RBI        (2+dev->base_addr)
+#define DATA_REG   (4+dev->base_addr)
+#define ADDR_REG   (6+dev->base_addr)
 #define CSR0       0 /* These are registers addressable through ADDR_REG */
 #define CSR1       1
 #define CSR2       2
 #define CSR3       3
-#define EBASE      (12+realdev->base_addr)
-#define RBSA       (14+realdev->base_addr)
+#define EBASE      (12+dev->base_addr)
+#define RBSA       (14+dev->base_addr)
 
 /* CSR bit definitions */
  
@@ -294,7 +399,7 @@ struct XmitMsgHeader { /*Transmit Message Descriptor */
 };
 struct InitBlock { /* LANCE Initialization Block */
   uword initMode;
-  uchar initAddr [ETH_ALEN]; /* Our Ethernet address */
+  uchar initAddr [ETH_ALEN];  /* Our Ethernet address */
   uchar initFilter [8];       /* Multicast filter. */
   uword initReceive [2];      /* Receive Ring Pointer. */
   uword initTransmit [2];     /* Transmit Ring Pointer. */
@@ -305,26 +410,11 @@ struct Lance {
   struct InitBlock initBlock;
 };
 
-#define lance ((struct Lance*)(realdev->mem_start))
-static struct device *realdev;
+#define lance ((struct Lance*)(dev->mem_start))
 
 #define transmitBufs 0x0800
 #define receiveBufs  0x1000
 
-/* unused static uword transmitHead=0;*/ /* ptr next packet to be filled by host. */
-static uword receiveHead=0;  /* ptr next packet to be filled by LANCE. */
-
-static uchar adapterInitOk=0;
-
-static void valToPort (uword v,uword reg) {
-  outw (reg,ADDR_REG);
-  inw  (ADDR_REG);
-  outw (v,DATA_REG);
-  inw  (DATA_REG);
-}
-
-static uchar niCsrValue=CSR_SHE|CSR_DUM|CSR_IEN|CSR_SWAP32;
-
 #ifdef DEPCA_DEBUG
 static void dump (char *p,int l) {
   int o=0;
@@ -374,260 +464,6 @@ static void T (uword w) {
 #define nointZero
 #endif
 
-static void iAm (struct device *dev) {
-  printk ("%s: DEPCA ",dev->name);
-}
-enum {translate_64k,translate_32k,translate_2k} xlatemode;
-
-/* returns a flat pointer to the lance's memory. a is the desired offset 
-   into lance's internal memory. the lance-window itself is remapped
-   according to the xlatemode required by the card. 
-*/
-static uchar xlcnt;
-static uchar* xlateAndMap (uword a) {
-  switch (xlatemode) {
-    case translate_64k: return (uchar *)lance+a;
-    case translate_32k: outb ((char)(a & 0x8000
-                        ?niCsrValue & ~CSR_SWAP32:niCsrValue),NI_CSR);
-                        return (uchar *)lance+(a & 0x7fff);
-    case translate_2k : outb ((uchar)(a>>11),RBI);
-                        return (uchar *)lance+0;
-  }
-}
-
-/*
- * Open/initialize the board.  This is called (in the current kernel)
- * sometime after booting when the 'config <dev->name>' program is run.
- *
- * This routine should set everything up anew at each open, even
- * registers that "should" only need to be set once at boot, so that
- * there is a non-reboot way to recover if something goes wrong.
- */
-static int depca_open(struct device *dev) {
-  adapter_init(dev, 1);
-  dev->tbusy = 0; /* Transmit busy...  */
-  dev->interrupt = 0;
-  return adapterInitOk?0:1;
-}
-
-/* The inverse routine to depca_open().  */
-static int depca_close(struct device *dev) {
-  dev->start = 0;
-  adapter_init(dev, 0);
-  return 0;
-}
-
-/* try to find available lance-xmit-buffer. return index to buffer
-   if fnd, else -1 */
-static int getXmitBuf (void) {
-  uword i;
-  for (i=0;i<TransmitBufCount;i++) {
-    if (!(lance->tramsmitHeaders[i].bufHi & XMIT_OWN))
-      return i;
-  }
-  return -1;
-}
-
-/* Copy a buffer to the adapter transmit page memory. Start sending. */
-static int depca_start_xmit(struct sk_buff *skb, struct device *dev) {
-  int xmitBufIdx;
-  unsigned char *buffer = (unsigned char *)(skb + 1);
-  /* If some higher layer thinks we've missed a tx-done interrupt, we are 
-     passed NULL. Caution: dev_tint() handles the cli()/sti() itself.  
-  */
-  nointPP;
-  T ('G');
-  if (skb == NULL) {
-  T ('S');
-    dev_tint(dev);
-    return 0;
-  }
-
-  /* For ethernet, fill in the header (hardware addresses) with an arp. */
-  if (!skb->arp  &&  dev->rebuild_header(skb + 1, dev)) {
-    skb->dev = dev;
-    arp_queue (skb);
-    return 0;
-  }
-
-  if ((xmitBufIdx=getXmitBuf())==-1) { /* Do timeouts, to avoid hangs. */
-    int tickssofar;
-    T ('R');
-    dev->tbusy=1; /* Dont know if this makes sense here. */
-    tickssofar = jiffies - dev->trans_start;
-    if (tickssofar >= 5) {
-      iAm (dev);
-      printk("transmit timed out (%d), driver error or Hardware (cable) problem\n",
-             tickssofar);
-      adapter_init(dev, 1);
-    }
-    return 1;
-  }
-  {struct XmitMsgHeader* lanceDest=&lance->tramsmitHeaders[xmitBufIdx];
-   lanceDest->bufHi &= 
-     ~(XMIT_ERR|XMIT_DEF|XMIT_1_RETRY|XMIT_RETRY);/* reset error indications. */
-   lanceDest->bufStat = 0; /* reset all error bits. */
-   lanceDest->negBufLen=
-        skb->len<RUNT?-RUNT:-(int)skb->len; /* store the negative of the cnt. */
-   memcpy (xlateAndMap (lanceDest->bufLo),
-            buffer,skb->len);/*store the packet. */
-   xlateAndMap (0);
-   lanceDest->bufHi |=XMIT_OWN; /* give it to the lance chip. */
-  }
-  dev->trans_start = jiffies;
-  if (TransmitBufCount==1)
-    dev->tbusy; /* in this case easy, otherwise have to scan all buffers. */
-                /* Dont know if this would be worth the info? */
-  T ('H');
-  outw (CSR0_INEA|CSR0_TDMD,DATA_REG);/* Inform LANCE to poll for a packet. */
-  T ('I');
-  if (skb->free) {
-    kfree_skb (skb, FREE_WRITE);
-  }
-  return 0;
-}
-static int ints=0;
-
-/* The typical workload of the driver: Handle the network interface interrupts.*/
-static void depca_interrupt(int reg_ptr) {
-  int irq = -(((struct pt_regs *)reg_ptr)->orig_eax+2);
-  struct device *dev = realdev;
-  uword         interrupts;
-
-  ints++;
-  if (ints>1) {
-    iAm (dev);
-    printk ("Things become interesting now, reentering isr %d. time\n",ints);
-  }
-  nointZero;
-  T ('J');
-  sti ();
-  outb (niCsrValue^=CSR_LED,NI_CSR); /* toggle led ... */
-  while (outw ((interrupts=inw(DATA_REG)) 
-         & ~(CSR0_TDMD|CSR0_STOP|CSR0_STRT|CSR0_INIT),DATA_REG),
-        interrupts & 0xff00) {
-    T (interrupts);
-    T ('U');
-    if (dev == NULL) {
-      printk ("?: DEPCA: irq %d for unknown device.\n", irq);
-    }
-    else {
-      dev->interrupt = 1;
-      if (dev->start == 0) {
-        if (interrupts & CSR0_IDON) {
-          dev->start=1; /* Now we have the GO from the card */
-          T ('Y');
-        }
-      }
-      else {
-        if (interrupts & CSR0_TINT) {
-          T ('K');
-          depca_tx_intr(dev);
-        }
-        if (interrupts & CSR0_RINT) {
-          T ('L');
-          depca_rx_intr(dev);
-        }
-        if (interrupts & CSR0_IDON) {
-          iAm (dev);
-          printk ("reinitialized\n"); 
-        }
-        if (interrupts & ~(CSR0_TINT|CSR0_RINT|CSR0_IDON|0xff)) {/* Card reports error */
-          iAm (dev);printk ("reports error %4.4x\n",interrupts);
-        }
-      }
-      dev->interrupt = 0;
-    }
-  }
-  T ('M');
-  --ints;
-  return;
-}
-
-/*
- * Do internal handshake: Transmitter done (of this page).
- * Also handle the case of a pending transmit page.
- */
-static void depca_tx_intr(struct device *dev) {
-  localstats->tx_packets++;
-  dev->tbusy = 0;
-  mark_bh(INET_BH);
-  T ('O');
-}
-
-/*
- * We have a good packet(s), get it/them out of the buffers.
- */
-static void depca_rx_intr(struct device *dev) {
-  struct sk_buff *skb=NULL;
-  int            size=0;
-  int            sksize;
-  int            fullrno=0,rno=receiveHead;
-  int            steps=0;
-  while (steps++<ReceiveBufCount) { /* this while loop fights against
-                                         lost interrupts. Maybe this 
-                                         makes no sense for linux? */
-    /* I left this algorythm in here because it was contained (in a
-       slightly different way) in the packet driver (depca.asm). If
-       no interrupts could get lost, and every received packet would
-       produce one interrupt, i would omit the while loop and only look at
-       the next buffer. */
-    if (!(lance->receiveHeaders[rno].bufHi & RCV_OWN)) {
-      steps=0;
-      fullrno=rno;
-      if (lance->receiveHeaders[rno].bufHi & RCV_ERR) {
-        T ('T');
-        localstats->rx_errors++; /* count all receive errors */
-      }
-      else {                                  /*  v reserved  v CRC */
-        size=(lance->receiveHeaders[rno].msgLen & 0x0fff)     - 4;
-        if ((size < RUNT  ||  size > GIANT)) {
-          iAm (dev);printk("Bogus packet size %d.\n", size);
-        }
-        else {
-          sksize = sizeof(struct sk_buff) + size;
-          if ((skb = (struct sk_buff*)kmalloc(sksize, GFP_ATOMIC)) == NULL) {
-            iAm (dev);printk("couldn't allocate a sk_buff of size %d.\n",
-                             sksize);
-            return;
-          }
-          else {
-            skb->lock = 0;
-            skb->mem_len = sksize;
-            skb->mem_addr = skb;
-            /* 'skb + 1' points to the start of sk_buff data area. */
-            memcpy ((unsigned char *)(skb +1),xlateAndMap 
-                    (lance->receiveHeaders[rno].bufLo),size);
-            xlateAndMap (0);
-            localstats->rx_packets++; /* count all receives */
-            T ('P');
-            if (dev_rint((uchar *)skb, 
-                size, IN_SKBUFF, dev)) /* ok, give it to them */ { 
-              /* these guys did'nt want this precious packet... */
-              iAm (dev);
-              printk ("dev_rint doesn't like packet: skb-address %8.8x size %d. Why ?\n",
-                      skb,size); 
-              return;
-            }
-            T ('Q');
-          }  
-        }
-      }
-      lance->receiveHeaders[rno].bufHi &= 
-        ~(RCV_ERR|RCV_FRAM|RCV_OFLO|RCV_CRC|RCV_BUF_ERR);
-      lance->receiveHeaders[rno].bufHi |= 
-        RCV_OWN; /* Give buffer back to lance */
-    }
-    rno=(rno+1) % ReceiveBufCount; /* advance to next */
-  }
-  receiveHead=(fullrno+1) % ReceiveBufCount;
-  /* If any worth-while packets have been received, dev_rint()
-   * has done a mark_bh(INET_BH) for us and will work on them
-   * when we get to the bottom-half routine.
-   */
-  return;
-}
-
 /* This routine tries to check, if the desired area is a rom, or
    a part of a rom starting elsewhere before p. 
 */
@@ -686,61 +522,6 @@ static int detectPort (uword p) { /* this is only minor voodo */
   return 0;
 }
 
-/* test to see if a board is located at dev->base_addr.               */
-/* setup to read first byte of ethernet address rom when successful.  */
-/* return 0 if not.                                                   */
-static int detectBoard (void) {
-  int ok;
-  outb (CSR_DUM,NI_CSR);
-  ok=detectPort(EBASE)||detectPort(EBASE+1);
-  outb (niCsrValue,NI_CSR);
-  return ok;
-}
-
-static int initialize (void) {
-  uword w;
-  int   laengerNicht;
-  long  offset=realdev->mem_start+((uchar*)&lance->initBlock-(uchar*)lance);
-  valToPort (CSR0_STOP,CSR0); /* reset the INIT bit. */
-  valToPort (2,CSR3);         /* write the bus config register. */
-  valToPort (offset & 0xffff,CSR1); /* write offset of init-block (low word) */
-  valToPort (offset >> 16,CSR2); /* write offset of init-block (high word). */
-  valToPort (CSR0_INEA|CSR0_STRT|CSR0_INIT,CSR0); /* reinit and restart. */
-  laengerNicht=jiffies+36; /* wait max 2 seconds */
-  w=inw (DATA_REG);
-  while (!(w & CSR0_IDON)) { 
-    if (jiffies>laengerNicht)
-      return 0;
-    w=inw (DATA_REG);           
-  }    
-  return 1;
-}
-
-static void initializeMulti (int v) {
-  int i;
-  for (i=0;i<8;i++)
-    lance->initBlock.initFilter[i]=v;
-}
-
-static int initializeNoMulti (int v) {
-  lance->initBlock.initMode=v;
-  return initialize();
-}
-
-
-enum receiveModeType {disaReceiveXmit,noMulticast,multicast,promiscuous};
-
-static int receiveMode (enum receiveModeType r) {
-  switch (r) {
-    case disaReceiveXmit: return initializeNoMulti (M_DRX|M_DTX);
-    case noMulticast    : initializeMulti (0);
-                          return initializeNoMulti (0);
-    case multicast      : initializeMulti (-1);
-                          return initializeNoMulti (0);
-    case promiscuous    : return initializeNoMulti (M_PROM);
-  }
-}
-
 static uword computeLog2 (uword n) {
   int res=-1;
   do {
@@ -750,9 +531,21 @@ static uword computeLog2 (uword n) {
   return res << 13;
 }
 
-static char *KnownSignatures [] = {"DEPCA","DE200",""};
+/* Start implementation of the depcadriver */
 
-static char *checkSignature (uchar * p) {
+void depcaDriver::iAm () {
+  printk ("%s: DEPCA ",dev->name);
+}
+
+void depcaDriver::valToPort (uword v,uword reg) {
+  outw (reg,ADDR_REG);
+  inw  (ADDR_REG);
+  outw (v,DATA_REG);
+  inw  (DATA_REG);
+}
+
+static char *KnownSignatures [] = {"DEPCA","DE200","DE100"};
+char *depcaDriver::checkSignature (uchar * p) {
 #ifdef ANY_SIGNATURE
   return "ANY";
 #else
@@ -774,15 +567,305 @@ static char *checkSignature (uchar * p) {
 #endif
 } 
 
-int depca_init(struct device *dev) {
+/* returns a flat pointer to the lance's memory. a is the desired offset 
+   into lance's internal memory. the lance-window itself is remapped
+   according to the xlatemode required by the card. 
+*/
+uchar* depcaDriver::xlateAndMap (uword a) {
+  switch (xlatemode) {
+    case translate_64k: return (uchar *)lance+a;
+    case translate_32k: outb ((char)(a & 0x8000
+                        ?niCsrValue & ~CSR_SWAP32:niCsrValue),NI_CSR);
+                        return (uchar *)lance+(a & 0x7fff);
+    case translate_2k : outb ((uchar)(a>>11),RBI);
+                        return (uchar *)lance+0;
+  }
+}
+
+int depcaDriver::open () {
+  adapterInit(1);
+  dev->tbusy = 0; /* Transmit busy...  */
+  dev->interrupt = 0;
+  return adapterInitOk?0:1;
+}
+
+int depcaDriver::close() {
+  dev->start = 0;
+  adapterInit(0);
+  return 0;
+}
+
+/* try to find available lance-xmit-buffer. return index to buffer
+   if fnd, else -1 */
+int depcaDriver::getXmitBuf (void) {
+  uword i;
+  for (i=0;i<TransmitBufCount;i++) {
+    if (!(lance->tramsmitHeaders[i].bufHi & XMIT_OWN))
+      return i;
+  }
+  return -1;
+}
+
+/* Copy a buffer to the adapter transmit page memory. Start sending. */
+int depcaDriver::startXmit(struct sk_buff *skb) {
+  int xmitBufIdx;
+  unsigned char *buffer = (unsigned char *)(skb + 1);
+  /* If some higher layer thinks we've missed a tx-done interrupt, we are 
+     passed NULL. Caution: dev_tint() handles the cli()/sti() itself.  
+  */
+  nointPP;
+  T ('G');
+  if (skb == NULL) {
+  T ('S');
+    dev_tint(dev);
+    return 0;
+  }
+
+  /* For ethernet, fill in the header (hardware addresses) with an arp. */
+  if (!skb->arp  &&  dev->rebuild_header(skb + 1, dev)) {
+    skb->dev = dev;
+    arp_queue (skb);
+    return 0;
+  }
+
+  if ((xmitBufIdx=getXmitBuf())==-1) { /* Do timeouts, to avoid hangs. */
+    int tickssofar;
+    T ('R');
+    dev->tbusy=1; /* Dont know if this makes sense here. */
+    tickssofar = jiffies - dev->trans_start;
+    if (tickssofar >= 5) {
+      iAm ();
+      printk("transmit timed out (%d), driver error or Hardware (cable) problem\n",
+             tickssofar);
+      adapterInit(1);
+    }
+    return 1;
+  }
+  {struct XmitMsgHeader* lanceDest=&lance->tramsmitHeaders[xmitBufIdx];
+   lanceDest->bufHi &= 
+     ~(XMIT_ERR|XMIT_DEF|XMIT_1_RETRY|XMIT_RETRY);/* reset error indications. */
+   lanceDest->bufStat = 0; /* reset all error bits. */
+   lanceDest->negBufLen=
+        skb->len<RUNT?-RUNT:-(int)skb->len; /* store the negative of the cnt. */
+   memcpy (xlateAndMap (lanceDest->bufLo),
+            buffer,skb->len);/*store the packet. */
+   xlateAndMap (0);
+   lanceDest->bufHi |=XMIT_OWN; /* give it to the lance chip. */
+  }
+  dev->trans_start = jiffies;
+  if (TransmitBufCount==1)
+    dev->tbusy; /* in this case easy, otherwise have to scan all buffers. */
+                /* Dont know if this would be worth the info? */
+  T ('H');
+  outw (CSR0_INEA|CSR0_TDMD,DATA_REG);/* Inform LANCE to poll for a packet. */
+  T ('I');
+  if (skb->free) {
+    kfree_skb (skb, FREE_WRITE);
+  }
+  return 0;
+}
+
+/* The typical workload of the driver: Handle the network interface interrupts.*/
+void depcaDriver::interrupt () {
+  uword         interrupts;
+  ints++;
+  if (ints>1) {
+    iAm ();
+    printk ("Things become interesting now, reentering isr %d. time\n",ints);
+  }
+  nointZero;
+  T ('J');
+  sti ();
+  outb (niCsrValue^=CSR_LED,NI_CSR); /* toggle led ... */
+  while (outw ((interrupts=inw(DATA_REG)) 
+         & ~(CSR0_TDMD|CSR0_STOP|CSR0_STRT|CSR0_INIT),DATA_REG),
+        interrupts & 0xff00) {
+    T (interrupts);
+    T ('U');
+    dev->interrupt = 1;
+    if (dev->start == 0) {
+      if (interrupts & CSR0_IDON) {
+        dev->start=1; /* Now we have the GO from the card */
+        T ('Y');
+      }
+    }
+    else {
+      if (interrupts & CSR0_TINT) {
+        T ('K');
+        txIntr();
+      }
+      if (interrupts & CSR0_RINT) {
+        T ('L');
+        rxIntr();
+      }
+      if (interrupts & CSR0_IDON) {
+        iAm ();
+        printk ("reinitialized\n"); 
+      }
+      if (interrupts & ~(CSR0_TINT|CSR0_RINT|CSR0_IDON|0xff)) {/* Card reports error */
+        iAm ();printk ("reports error %4.4x\n",interrupts);
+      }
+    }
+    dev->interrupt = 0;
+  }
+  T ('M');
+  --ints;
+  return;
+}
+
+/*
+ * Do internal handshake: Transmitter done (of this page).
+ * Also handle the case of a pending transmit page.
+ */
+void depcaDriver::txIntr() {
+  localstats->tx_packets++;
+  dev->tbusy = 0;
+  mark_bh(INET_BH);
+  T ('O');
+}
+
+/*
+ * We have a good packet(s), get it/them out of the buffers.
+ */
+void depcaDriver::rxIntr() {
+  struct sk_buff *skb=NULL;
+  int            size=0;
+  int            sksize;
+  int            fullrno=0,rno=receiveHead;
+  int            steps=0;
+  while (steps++<ReceiveBufCount) { /* this while loop fights against
+                                         lost interrupts. Maybe this 
+                                         makes no sense for linux? */
+    /* I left this algorythm in here because it was contained (in a
+       slightly different way) in the packet driver (depca.asm). If
+       no interrupts could get lost, and every received packet would
+       produce one interrupt, i would omit the while loop and only look at
+       the next buffer. */
+    if (!(lance->receiveHeaders[rno].bufHi & RCV_OWN)) {
+      steps=0;
+      fullrno=rno;
+      if (lance->receiveHeaders[rno].bufHi & RCV_ERR) {
+        T ('T');
+        localstats->rx_errors++; /* count all receive errors */
+      }
+      else {                                  /*  v reserved  v CRC */
+        size=(lance->receiveHeaders[rno].msgLen & 0x0fff)     - 4;
+        if ((size < RUNT  ||  size > GIANT)) {
+          iAm ();printk("Bogus packet size %d.\n", size);
+        }
+        else {
+          sksize = sizeof(struct sk_buff) + size;
+          if ((skb = (struct sk_buff*)kmalloc(sksize, GFP_ATOMIC)) == NULL) {
+            iAm ();printk("couldn't allocate a sk_buff of size %d.\n",
+                             sksize);
+            return;
+          }
+          else {
+            skb->lock = 0;
+            skb->mem_len = sksize;
+            skb->mem_addr = skb;
+            /* 'skb + 1' points to the start of sk_buff data area. */
+            memcpy ((unsigned char *)(skb +1),xlateAndMap 
+                    (lance->receiveHeaders[rno].bufLo),size);
+            xlateAndMap (0);
+            localstats->rx_packets++; /* count all receives */
+            T ('P');
+            if (dev_rint((uchar *)skb, 
+                size, IN_SKBUFF, dev)) /* ok, give it to them */ { 
+              /* these guys did'nt want this precious packet... */
+              iAm ();
+              printk ("dev_rint doesn't like packet: skb-address %8.8x size %d. Why ?\n",
+                      skb,size); 
+              return;
+            }
+            T ('Q');
+          }  
+        }
+      }
+      lance->receiveHeaders[rno].bufHi &= 
+        ~(RCV_ERR|RCV_FRAM|RCV_OFLO|RCV_CRC|RCV_BUF_ERR);
+      lance->receiveHeaders[rno].bufHi |= 
+        RCV_OWN; /* Give buffer back to lance */
+    }
+    rno=(rno+1) % ReceiveBufCount; /* advance to next */
+  }
+  receiveHead=(fullrno+1) % ReceiveBufCount;
+  /* If any worth-while packets have been received, dev_rint()
+   * has done a mark_bh(INET_BH) for us and will work on them
+   * when we get to the bottom-half routine.
+   */
+  return;
+}
+
+/* test to see if a board is located at dev->base_addr.               */
+/* setup to read first byte of ethernet address rom when successful.  */
+/* return 0 if not.                                                   */
+int depcaDriver::detectBoard (void) {
+  int ok;
+  outb (CSR_DUM,NI_CSR);
+  ok=detectPort(EBASE)||detectPort(EBASE+1);
+  outb (niCsrValue,NI_CSR);
+  return ok;
+}
+
+int depcaDriver::initialize (void) {
+  uword w;
+  int   laengerNicht;
+  long  offset=dev->mem_start+((uchar*)&lance->initBlock-(uchar*)lance);
+  valToPort (CSR0_STOP,CSR0); /* reset the INIT bit. */
+  valToPort (2,CSR3);         /* write the bus config register. */
+  valToPort (offset & 0xffff,CSR1); /* write offset of init-block (low word) */
+  valToPort (offset >> 16,CSR2); /* write offset of init-block (high word). */
+  valToPort (CSR0_INEA|CSR0_STRT|CSR0_INIT,CSR0); /* reinit and restart. */
+  laengerNicht=jiffies+36; /* wait max 2 seconds */
+  w=inw (DATA_REG);
+  while (!(w & CSR0_IDON)) { 
+    if (jiffies>laengerNicht)
+      return 0;
+    w=inw (DATA_REG);           
+  }    
+  return 1;
+}
+
+void depcaDriver::initializeMulti (int v) {
+  int i;
+  for (i=0;i<8;i++)
+    lance->initBlock.initFilter[i]=v;
+}
+
+int depcaDriver::initializeNoMulti (int v) {
+  lance->initBlock.initMode=v;
+  return initialize();
+}
+
+int depcaDriver::receiveMode (enum receiveModeType r) {
+  switch (r) {
+    case disaReceiveXmit: return initializeNoMulti (M_DRX|M_DTX);
+    case noMulticast    : initializeMulti (0);
+                          return initializeNoMulti (0);
+    case multicast      : initializeMulti (-1);
+                          return initializeNoMulti (0);
+    case promiscuous    : return initializeNoMulti (M_PROM);
+  }
+}
+
+depcaDriver::depcaDriver (struct device *_dev) 
+  : receiveHead(0), transmitHead(0), adapterInitOk (0),
+    dev (_dev),
+    niCsrValue(CSR_SHE|CSR_DUM|CSR_IEN|CSR_SWAP32),
+    xlatemode (translate_64k),
+    ints (0),
+    localstats(_dev->priv) {
+  // nothing, only used to init the var's
+}
+
+int depcaDriver::init () {
   uchar nic;
-  uchar * ptr;
+  uchar *ptr;
   int boardfound=1;
   int havemem=0;
   int error=1; /* Our default return */
 
-  realdev = dev;
-
   BlaBla ((version));
   if (dev->base_addr!=0) {   /* The ioaddress is hardcoded, so check if its ok */
     if (!detectBoard()) {
@@ -804,7 +887,7 @@ int depca_init(struct device *dev) {
     if (((dev->mem_start & 0x7ff) != 0)  /* must be on 2K boundary. */
        || ((dev->mem_start & 0x80000) == 0) /* must be in upper 512K. */
        || ((dev->mem_start & 0xfff00000) != 0)) { /*high word must be zero.*/
-      iAm (dev);printk ("bad shared memory-address %x\n",dev->mem_start);
+      iAm ();printk ("bad shared memory-address %x\n",dev->mem_start);
     }
     else { /* The following is a kind of voodo mixed with black digital equip-
               ment magic and salted with the art of predicting the future out 
@@ -838,7 +921,7 @@ int depca_init(struct device *dev) {
         outw (0xf000,RBSA); /* put our memory somewhere it can't be. */
         if (isRom ((uchar*)dev->mem_start,0x800)
           ||isRam ((uchar*)dev->mem_start,0x800)) {
-          iAm (dev);printk ("remap not possible due to address in use %x\n",dev->mem_start);
+          iAm ();printk ("remap not possible due to address in use %x\n",dev->mem_start);
         }
         else { /* Weder Fisch noch Fleisch */
             outw ((uword)(dev->mem_start&0xffff),
@@ -885,12 +968,12 @@ int depca_init(struct device *dev) {
            be done is only to tell/ask the kernel, if we are allowed to do
            so later ... */
         if (irqaction (dev->irq, &depca_sigaction)) { 
-          iAm (dev);printk ("unable to get IRQ %d\n", dev->irq);
+          iAm ();printk ("unable to get IRQ %d\n", dev->irq);
         }
         else {
           char *wsize[3]={"64","32","2"};
           error=0; /* uff */
-          iAm (dev);
+          iAm ();
           printk ("ethernet adapter (%sK, IOBase %x, MemStart %x, Ethernet %2.2X", 
                    wsize[xlatemode],dev->base_addr,dev->mem_start,dev->dev_addr[0]);
           for (i = 1; i < ETH_ALEN; i++)
@@ -903,10 +986,9 @@ int depca_init(struct device *dev) {
   return error;
 }
 
-static void adapter_init(struct device *dev, int startp) {
+void depcaDriver::adapterInit(int startp) {
   adapterInitOk=1; /* assume we succeed */
   cli();
-  realdev = dev;
   valToPort (CSR0_STOP,CSR0);
   if (startp) {
     int  i;
